Imports System.Runtime.InteropServices
Imports System.Collections.Generic

Imports Excel = Microsoft.Office.Interop.Excel
Imports Word = Microsoft.Office.Interop.Word
Imports PowerPoint = Microsoft.Office.Interop.PowerPoint
Imports Microsoft.Office.Interop.Excel

'Add-in Express Add-in Module
<GuidAttribute("AB07BADE-56F7-414B-ACA0-D3E2DDAABCCB"), ProgIdAttribute("MyAddin1VB.AddinModule")>
Partial Public Class AddinModule
    Inherits AddinExpress.MSO.ADXAddinModule
    Implements Logging.ILogPaneManager

    Dim myWorksheetEventClass As ExcelWorksheetEventsClass1 = Nothing

    Public Sub New()
        System.Windows.Forms.Application.EnableVisualStyles()
        InitializeComponent()
        AddHandler Logging.Log.Instance.NewData, AddressOf Log_NewData
    End Sub

    Public Const showPanesAtStartup As Boolean = True
    Const copyRibbonXmlToClipboard As Boolean = False

#Region " Add-in Express automatic code "

    'Required by Add-in Express - do not modify
    'the methods within this region

    Public Overrides Function GetContainer() As System.ComponentModel.IContainer
        If components Is Nothing Then
            components = New System.ComponentModel.Container
        End If
        GetContainer = components
    End Function

    <ComRegisterFunctionAttribute()>
    Public Shared Sub AddinRegister(ByVal t As Type)
        AddinExpress.MSO.ADXAddinModule.ADXRegister(t)
    End Sub

    <ComUnregisterFunctionAttribute()>
    Public Shared Sub AddinUnregister(ByVal t As Type)
        AddinExpress.MSO.ADXAddinModule.ADXUnregister(t)
    End Sub

    Public Overrides Sub UninstallControls()
        MyBase.UninstallControls()
    End Sub
#End Region

#Region "Useful propertis"
    Public Shared Shadows ReadOnly Property CurrentInstance() As AddinModule
        Get
            Return CType(AddinExpress.MSO.ADXAddinModule.CurrentInstance, AddinModule)
        End Get
    End Property

    Public ReadOnly Property ExcelApp() As Excel._Application
        Get
            Return CType(HostApplication, Excel._Application)
        End Get
    End Property

    Public ReadOnly Property WordApp() As Word._Application
        Get
            Return CType(HostApplication, Word._Application)
        End Get
    End Property

    Public ReadOnly Property PowerPointApp() As PowerPoint._Application
        Get
            Return CType(HostApplication, PowerPoint._Application)
        End Get
    End Property
#End Region

#Region "Add-in events"
    Private Sub AddinModule_AddinStartupComplete(ByVal sender As Object, ByVal e As System.EventArgs) Handles MyBase.AddinStartupComplete
        If Me.HostType = AddinExpress.MSO.ADXOfficeHostApp.ohaExcel Then
            myWorksheetEventClass = New ExcelWorksheetEventsClass1(Me)
        End If
    End Sub
    Dim isShutdown As Boolean = False
    Private Sub AddinModule_AddinBeginShutdown(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.AddinBeginShutdown
        isShutdown = True
        If Me.HostType = AddinExpress.MSO.ADXOfficeHostApp.ohaExcel Then
            If myWorksheetEventClass.IsConnected Then myWorksheetEventClass.RemoveConnection()
        End If
    End Sub
#End Region

#Region "Host application events"
    Private Sub WindowActivate(ByVal sender As Object, ByVal hostObj As Object,
                         ByVal window As Object) _
                         Handles AdxWordEvents.WindowActivate, AdxExcelEvents.WindowActivate, AdxPowerPointEvents.WindowActivate
        Logging.Log.Add("AddinModule.WindowActivate", HostType.ToString())

        Select Case Me.HostType
            Case AddinExpress.MSO.ADXOfficeHostApp.ohaExcel

                If myWorksheetEventClass.IsConnected Then
                    myWorksheetEventClass.RemoveConnection()
                End If

                Dim sheet As Object = (TryCast(window, Excel.Window)).ActiveSheet
                myWorksheetEventClass.ConnectTo(sheet, eventClassReleasesComObject:=True)
            Case AddinExpress.MSO.ADXOfficeHostApp.ohaWord
            Case AddinExpress.MSO.ADXOfficeHostApp.ohaPowerPoint
            Case Else
                Logging.Log.Add("AddinModule.WindowActivate", Me.AddinName & " doesn't support " + Me.HostName)
        End Select
    End Sub

    Private Sub Deactivate(ByVal sender As Object, ByVal hostObj As Object,
                           ByVal window As Object) _
                           Handles AdxWordEvents.WindowDeactivate, AdxExcelEvents.WindowDeactivate, AdxPowerPointEvents.WindowDeactivate
        Select Case Me.HostType
            Case AddinExpress.MSO.ADXOfficeHostApp.ohaExcel
                If myWorksheetEventClass.IsConnected Then myWorksheetEventClass.RemoveConnection()
            Case AddinExpress.MSO.ADXOfficeHostApp.ohaWord
            Case AddinExpress.MSO.ADXOfficeHostApp.ohaPowerPoint
            Case Else
                Logging.Log.Add("AddinModule.WindowActivate", Me.AddinName & " doesn't support " + Me.HostName)
        End Select
    End Sub

    Private Sub AdxExcelEvents_SheetActivate(ByVal sender As System.Object, ByVal hostObj As System.Object) Handles AdxExcelEvents.SheetActivate
        Logging.Log.Add("AddinModule.adxExcelEvents_SheetActivate", "")
    End Sub

    Private Sub AdxExcelEvents_SheetSelectionChange(ByVal sender As System.Object, ByVal sheet As System.Object, ByVal range As System.Object) Handles AdxExcelEvents.SheetSelectionChange
        Logging.Log.Add("AddinModule.adxExcelEvents_SheetSelectionChange", ContextInfo.GetContextInfo(Me.ExcelApp, Me.HostType).ToString())
    End Sub

    Private Sub AdxExcelEvents_WorkbookActivate(ByVal sender As System.Object, ByVal hostObj As System.Object) Handles AdxExcelEvents.WorkbookActivate
        Logging.Log.Add("AddinModule.adxExcelEvents_WorkbookActivate", "")
    End Sub

    Private Sub AdxWordEvents_WindowSelectionChange(ByVal sender As System.Object, ByVal hostObj As System.Object) Handles AdxWordEvents.WindowSelectionChange
        Logging.Log.Add("AddinModule.adxWordEvents_WindowSelectionChange", ContextInfo.GetContextInfo(Me.WordApp, Me.HostType).ToString())
    End Sub

    Private Sub AdxPowerPointEvents_SlideSelectionChanged(ByVal sender As System.Object, ByVal hostObj As System.Object) Handles AdxPowerPointEvents.SlideSelectionChanged
        Logging.Log.Add("AddinModule.adxPowerPointEvents_SlideSelectionChanged", "")
    End Sub

    Private Sub AdxKeyboardShortcut1_Action(ByVal sender As Object) Handles AdxKeyboardShortcut1.Action
        Dim window As Object = Nothing

        Try
            Dim message As String = String.Format("You've pressed '{0}'", (TryCast(sender, AddinExpress.MSO.ADXKeyboardShortcut)).ShortcutText)
            System.Windows.Forms.MessageBox.Show(message, AddinName)

            ' late binding, Option Strict On
            window = Me.HostApplication.GetType().InvokeMember("ActiveWindow", System.Reflection.BindingFlags.GetProperty, Nothing, Me.HostApplication, Nothing)
            ' late binding, Option Strict Off
            ' Window = Me.HostApplication.ActiveWindow()


            Dim contextInfo As ContextInfo = ContextInfo.GetContextInfo(Me.HostType, window)
            Logging.Log.Add("AddinModule.adxKeyboardShortcut1_Action", message & "; " & contextInfo.ToString())
        Catch ex As Exception
            If Me.HostType <> AddinExpress.MSO.ADXOfficeHostApp.ohaWord Then
                Logging.Log.Add("AddinModule.AdxKeyboardShortcut1_Action", ex)
            Else
                Logging.Log.Add("AddinModule.AdxKeyboardShortcut1_Action", "No windows? + " + ex.ToString())
            End If
        Finally
            If window IsNot Nothing Then Marshal.ReleaseComObject(window)
        End Try
    End Sub
#End Region

#Region "CommandBar controls"
    Private Sub CommandBarButton_CommonClickHandler(ByVal sender As Object) _
        Handles AdxCommandBarButton1.Click, AdxCommandBarButton3.Click, AdxCommandBarButton4.Click, AdxCommandBarButton5.Click, AdxCommandBarButton6.Click

        Dim buttonCaption As String = (TryCast(sender, AddinExpress.MSO.ADXCommandBarButton)).Caption
        Dim contextInfo As ContextInfo = contextInfo.GetContextInfo(Me.HostApplication, Me.HostType)
        Dim message As String = String.Format("Button '{0}' clicked." & vbLf & vbLf & "Context:" & vbLf & "{1}" & vbLf & vbLf & "Host:" & vbLf & "{2}", buttonCaption, contextInfo, contextInfo.GetHostSummary())
        System.Windows.Forms.MessageBox.Show(message, "CommandBarButton_CommonClickHandler")
    End Sub
#End Region

#Region "Ribbon events of the add-in module"
    Private Sub AddinModule_OnRibbonBeforeCreate(sender As Object, ribbonId As String) Handles MyBase.OnRibbonBeforeCreate
        Me.AdxRibbonButtonPaneVisibilityInfo.Visible = showPanesAtStartup
    End Sub
    Private Sub AddinModule_OnRibbonBeforeLoad(sender As Object, e As AddinExpress.MSO.ADXRibbonBeforeLoadEventArgs) Handles MyBase.OnRibbonBeforeLoad
        If copyRibbonXmlToClipboard Then
            System.Windows.Forms.Clipboard.SetText(e.Xml)
        End If
    End Sub
#End Region

#Region "Events of Ribbon controls"
    Private Sub RibbonButton_CommonClickHandler(ByVal sender As Object, ByVal control As AddinExpress.MSO.IRibbonControl, ByVal pressed As Boolean) Handles AdxRibbonButton2.OnClick, AdxRibbonButton1.OnClick
        Dim buttonCaption As String = (TryCast(sender, AddinExpress.MSO.ADXRibbonButton)).Caption
        Dim context As Object = control.Context
        Dim contextInfo As ContextInfo = contextInfo.GetContextInfo(Me.HostType, context)
        If context IsNot Nothing Then Marshal.ReleaseComObject(context)
        context = Nothing
        Dim message As String = String.Format("Button '{0}' clicked." & vbLf & vbLf & "Context:" & vbLf & "{1}" & vbLf & vbLf & "Host:" & vbLf & "{2}", buttonCaption, contextInfo, contextInfo.GetHostSummary())
        System.Windows.Forms.MessageBox.Show(message, AddinName)
        Logging.Log.Add("AddinModule.RibbonButton_CommonClickHandler", message)
    End Sub
    Private Sub AdxRibbonButton3_OnClick(ByVal sender As System.Object, ByVal control As AddinExpress.MSO.IRibbonControl, ByVal pressed As System.Boolean) Handles AdxRibbonButton3.OnClick
        Me.AdxRibbonButton4.Visible = Not AdxRibbonButton4.Visible
        System.Windows.Forms.MessageBox.Show("This has changed the visibility of My Menu Button 2. Please click Ok and open the menu anew.")
    End Sub
    Private Sub AdxRibbonButton4_OnClick(ByVal sender As System.Object, ByVal control As AddinExpress.MSO.IRibbonControl, ByVal pressed As System.Boolean) Handles AdxRibbonButton4.OnClick
        System.Windows.Forms.MessageBox.Show("You've just clicked My Menu Button 2!")
    End Sub
    Private Sub AdxBackstageRegularButton1_OnClick(ByVal sender As System.Object, ByVal control As AddinExpress.MSO.IRibbonControl) Handles AdxBackstageRegularButton1.OnClick
        System.Windows.Forms.MessageBox.Show("You've clicked the " & control.Id & " button. ")
    End Sub
#End Region

#Region "Dynamic menu"

    Private db As Dictionary(Of Integer, String) = New Dictionary(Of Integer, String)()
    Private counter As Integer = 0

    Private Sub AdxRibbonButtonAddRecord_OnClick(sender As Object, control As AddinExpress.MSO.IRibbonControl, pressed As Boolean) Handles AdxRibbonButtonAddRecord.OnClick
        Dim id As Integer = Math.Min(System.Threading.Interlocked.Increment(counter), counter - 1)
        Dim context As Object = control.Context
        Dim contextInfo As ContextInfo = contextInfo.GetContextInfo(Me.HostType, context)
        Dim windowCaption As String = contextInfo.WindowCaption
        If context IsNot Nothing Then Marshal.ReleaseComObject(context)
        context = Nothing
        Dim time As String = System.DateTime.Now.ToLongTimeString()
        Dim value As String = String.Format("{0} Window={1}", time, windowCaption)
        If db.Count = 0 Then db.Add(id, "separator")
        id = Math.Min(System.Threading.Interlocked.Increment(counter), counter - 1)
        db.Add(id, value)
        Me.AdxRibbonMenuDynamic.Caption = String.Format("Menu ({0} items)", db.Count)
    End Sub
    Private Sub AdxRibbonButtonClearRecords_OnClick(sender As Object, control As AddinExpress.MSO.IRibbonControl, pressed As Boolean) Handles AdxRibbonButtonClearRecords.OnClick
        db.Clear()
        Me.AdxRibbonMenuDynamic.Caption = String.Format("Menu ({0} items)", 0)
    End Sub
    Private Sub AdxRibbonMenuDynamic_OnCreate(sender As Object, e As AddinExpress.MSO.ADXRibbonCreateMenuEventArgs) Handles AdxRibbonMenuDynamic.OnCreate
        e.Clear()

        For Each pair As KeyValuePair(Of Integer, String) In db

            If pair.Value.ToLower() = "separator" Then
                Dim separator As AddinExpress.MSO.ADXRibbonMenuSeparator = New AddinExpress.MSO.ADXRibbonMenuSeparator(Me.components)
                separator.Id = "newControl" & pair.Key.ToString()
                separator.Ribbons = Me.AdxRibbonMenuDynamic.Ribbons
                Me.AdxRibbonMenuDynamic.Controls.Add(separator)
                e.AddControl(separator)
            Else
                Dim newButton As AddinExpress.MSO.ADXRibbonButton = New AddinExpress.MSO.ADXRibbonButton(Me.components)
                newButton.Id = "newControl" & pair.Key.ToString()
                newButton.Caption = pair.Value
                newButton.Ribbons = Me.AdxRibbonMenuDynamic.Ribbons
                AddHandler newButton.OnClick, AddressOf RibbonButton_CommonClickHandler
                e.AddControl(newButton)
            End If
        Next
    End Sub
#End Region

#Region "Pane-related"
    Private showButtonInfoShown As Boolean = False
    Private Sub AdxRibbonButtonPaneVisibilityInfo_OnClick(ByVal sender As Object, ByVal control As AddinExpress.MSO.IRibbonControl, ByVal pressed As Boolean) Handles AdxRibbonButtonPaneVisibilityInfo.OnClick
        System.Windows.Forms.MessageBox.Show(paneDefaultVisibilityMessage, AddinName)
        AdxRibbonButtonPaneVisibilityInfo.Visible = False
        Logging.Log.Add("AddinModule.adxRibbonButtonPaneVisibilityInfo_OnClick", "paneDefaultVisibilityMessage shown")
    End Sub
    Private Sub AdxRibbonButtonShowPane_OnClick(ByVal sender As Object, ByVal control As AddinExpress.MSO.IRibbonControl, ByVal pressed As Boolean) Handles AdxRibbonButtonShowPane.OnClick
        Dim context As Object = control.Context
        Dim pane As Logging.ILogPane = GetPane(Me.HostType, context)

        If pane IsNot Nothing Then
            ShowInfoMessage()
            pane.Show()
        End If

        If context IsNot Nothing Then Marshal.ReleaseComObject(context)
    End Sub
    Private Sub ShowInfoMessage()
        If Not showButtonInfoShown Then
            showButtonInfoShown = True
            System.Windows.Forms.MessageBox.Show(paneChangeVisibilityMessage, AddinName)
            Logging.Log.Add("AddinModule.ShowInfoMessage", "paneChangeVisibilityMessage shown")
        End If
    End Sub
    Private Sub AdxRibbonButtonHidePane_OnClick(ByVal sender As Object, ByVal control As AddinExpress.MSO.IRibbonControl, ByVal pressed As Boolean) Handles AdxRibbonButtonHidePane.OnClick
        Dim context As Object = control.Context
        Dim pane As Logging.ILogPane = GetPane(Me.HostType, context)

        If pane IsNot Nothing Then
            ShowInfoMessage()
            pane.Hide()
        End If

        If context IsNot Nothing Then Marshal.ReleaseComObject(context)
    End Sub
    Private Sub AdxRibbonButtonShowPane_PropertyChanging(ByVal sender As Object, ByVal e As AddinExpress.MSO.ADXRibbonPropertyChangingEventArgs) Handles AdxRibbonButtonShowPane.PropertyChanging
        If e.PropertyType = AddinExpress.MSO.ADXRibbonControlPropertyType.Visible Then
            Logging.Log.Add("AddinModule.adxRibbonButtonShowPane_PropertyChanging", "ADXRibbonControlPropertyType.Visible")
            Dim result As Boolean = False
            Dim context As Object = e.Context
            Dim pane As Logging.ILogPane = GetPane(Me.HostType, context)

            If pane IsNot Nothing Then
                result = pane.IsPaneShown
            End If

            e.Value = Not result
        End If
    End Sub
    Private Sub AdxRibbonButtonHidePane_PropertyChanging(ByVal sender As Object, ByVal e As AddinExpress.MSO.ADXRibbonPropertyChangingEventArgs) Handles AdxRibbonButtonHidePane.PropertyChanging
        If e.PropertyType = AddinExpress.MSO.ADXRibbonControlPropertyType.Visible Then
            Logging.Log.Add("AddinModule.adxRibbonButtonHidePane_PropertyChanging", "ADXRibbonControlPropertyType.Visible")
            Dim result As Boolean = False
            Dim context As Object = e.Context
            Dim pane As Logging.ILogPane = GetPane(Me.HostType, context)

            If pane IsNot Nothing Then
                result = pane.IsPaneShown
            End If

            e.Value = result
        End If
    End Sub
    Private Function GetPane(ByVal host As AddinExpress.MSO.ADXOfficeHostApp, ByVal contextWindow As Object) As Logging.ILogPane
        If contextWindow Is Nothing Then
            Logging.Log.Add("AddinModule.GetPane", "context not provided")
            Return Nothing
        End If

        Dim pane As Logging.ILogPane = Nothing

        Select Case host
            Case AddinExpress.MSO.ADXOfficeHostApp.ohaExcel
                pane = TryCast(AdxExcelTaskPanesCollectionItem1.TaskPaneInstance, Logging.ILogPane)
            Case AddinExpress.MSO.ADXOfficeHostApp.ohaWord
                pane = TryCast(AdxWordTaskPanesCollectionItem1.CurrentTaskPaneInstance, Logging.ILogPane)
            Case AddinExpress.MSO.ADXOfficeHostApp.ohaPowerPoint
                pane = TryCast(AdxPowerPointTaskPanesCollectionItem1.TaskPaneInstance, Logging.ILogPane)
            Case Else
                Logging.Log.Add("AddinModule.GetPane", "non supported host type: " & host.ToString())
        End Select

        Return pane
    End Function
    Private Sub adxExcelTaskPanesManager1_ADXBeforeTaskPaneInstanceCreate(ByVal sender As Object, ByVal e As AddinExpress.XL.ADXBeforeTaskPaneInstanceCreateEventArgs) Handles AdxExcelTaskPanesManager1.ADXBeforeTaskPaneInstanceCreate
        Logging.Log.Add("AddinModule.adxExcelTaskPanesManager1_ADXBeforeTaskPaneInstanceCreate", "let a new pane instance create")
    End Sub
    Private Sub adxPowerPointTaskPanesManager1_ADXBeforeTaskPaneInstanceCreate(ByVal sender As Object, ByVal e As AddinExpress.PP.ADXBeforeTaskPaneInstanceCreateEventArgs) Handles AdxPowerPointTaskPanesManager1.ADXBeforeTaskPaneInstanceCreate
        Logging.Log.Add("AddinModule.adxPowerPointTaskPanesManager1_ADXBeforeTaskPaneInstanceCreate", "let a new pane instance create")
    End Sub
    Private Sub adxWordTaskPanesManager1_ADXBeforeTaskPaneInstanceCreate(ByVal sender As Object, ByVal e As AddinExpress.WD.ADXBeforeTaskPaneInstanceCreateEventArgs) Handles AdxWordTaskPanesManager1.ADXBeforeTaskPaneInstanceCreate
        Logging.Log.Add("AddinModule.adxWordTaskPanesManager1_ADXBeforeTaskPaneInstanceCreate", "let a new pane instance create")
    End Sub

#End Region

#Region "Log-related"
    Private Sub Log_NewData(ByVal sender As Object, ByVal e As Logging.LogRecord)
        System.Diagnostics.Debug.WriteLine(Me.AddinName + ": " + e.ToString())
        RaiseEvent NewLogData(TryCast(Me, Logging.ILogPaneManager), e)
    End Sub

    Friend Event NewLogData As EventHandler(Of Logging.LogRecord) Implements Logging.ILogPaneManager.NewLogData

    Friend Function GetRecords() As List(Of Logging.LogRecord) Implements Logging.ILogPaneManager.GetRecords
        Return Logging.Log.Instance.GetData()
    End Function
    Public Sub PaneVisibilityChanged(logPane As Logging.ILogPane) Implements Logging.ILogPaneManager.PaneVisibilityChanged
        If isShutdown Then Return
        Me.AdxRibbonButtonShowPane.Invalidate()
        Me.AdxRibbonButtonHidePane.Invalidate()
    End Sub
#End Region

    Const paneChangeVisibilityMessage As String = "This will hide or show the pane instance associated with this window. Opening another window will create a new pane instance."
    Const paneDefaultVisibilityMessage As String = "To change the default visibility of the Add-in Express panes that the add-in creates, set showPanesAtStartup=false in the code of the add-in module and compile the project"

End Class
