﻿Imports System
Imports System.Runtime.InteropServices
Imports AddinExpress.MSO
Imports Excel = Microsoft.Office.Interop.Excel
Imports Word = Microsoft.Office.Interop.Word
Imports PowerPoint = Microsoft.Office.Interop.PowerPoint


Class ContextInfo
        Public Property WindowType As String
        Public Property WindowCaption As String
        Public Property ContextDataString As String

        Public Shared Function GetContextInfo(ByVal hostApplication As Object, ByVal host As ADXOfficeHostApp) As ContextInfo
            If hostApplication Is Nothing Then
                Return New ContextInfoUnknown()
            End If

            Select Case host
                Case ADXOfficeHostApp.ohaExcel
                    Return New ContextInfoExcel(TryCast(hostApplication, Excel._Application))
                Case ADXOfficeHostApp.ohaWord
                    Return New ContextInfoWord(TryCast(hostApplication, Word._Application))
                Case ADXOfficeHostApp.ohaPowerPoint
                    Return New ContextInfoPowerPoint(TryCast(hostApplication, PowerPoint._Application))
                Case Else
                    Return New ContextInfoUnknown()
            End Select
        End Function

        Public Shared Function GetContextInfo(ByVal host As ADXOfficeHostApp, ByVal context As Object) As ContextInfo
            If context Is Nothing Then
                Logging.Log.Add("ContextInfo.ctor", "No context provided")
                Return New ContextInfoUnknown()
            End If

            Dim contextInfo As ContextInfo = Nothing
            Dim isOkay As Boolean = False

            Select Case host
                Case ADXOfficeHostApp.ohaExcel

                    If TypeOf context Is Excel.Window Then
                        contextInfo = New ContextInfoExcel(TryCast(context, Excel.Window))
                        isOkay = True
                    End If

                Case ADXOfficeHostApp.ohaWord

                    If TypeOf context Is Word.Window Then
                        contextInfo = New ContextInfoWord(TryCast(context, Word.Window))
                        isOkay = True
                    End If

                Case ADXOfficeHostApp.ohaPowerPoint

                    If TypeOf context Is PowerPoint.DocumentWindow Then
                        contextInfo = New ContextInfoPowerPoint(TryCast(context, PowerPoint.DocumentWindow))
                        isOkay = True
                    End If

                Case Else
            End Select

            If Not isOkay Then
                Dim contextTypeName As String = Microsoft.VisualBasic.Information.TypeName(context)
                Dim message As String = String.Format("{0} - non-supported context type", contextTypeName)
                Logging.Log.Add("ContextInfo.GetSummaryFromContexObject", New System.Exception(message))
                contextInfo = New ContextInfoUnknown()
            End If

            Return contextInfo
        End Function

        Public Overrides Function ToString() As String
            Return String.Format("Window: {0}" & vbLf & "Caption: {1}" & vbLf & "{2}", WindowType, WindowCaption, ContextDataString)
        End Function

        Public Shared Function GetHostSummary() As String
            Dim addinModule As MyAddin1.AddinModule = MyAddin1.AddinModule.CurrentInstance
            Dim s As String = String.Format("HostBitness={0}" & vbLf & "HostMajorVersion={1}" & vbLf & "HostName={2}" & vbLf & "HostStartMode={3}" & vbLf & "HostType={4}" & vbLf & "HostVersion={5}", addinModule.HostBitness, addinModule.HostMajorVersion, addinModule.HostName, addinModule.HostStartMode, addinModule.HostType, addinModule.HostVersion)
            Return s
        End Function
    End Class

Class ContextInfoExcel
    Inherits ContextInfo

    Public Sub New(ByVal hostApp As Excel._Application)
        Dim excelApp As Excel._Application = TryCast(hostApp, Excel._Application)
        ContextDataString = GetSummaryFromApplication(excelApp)
    End Sub

    Public Sub New(ByVal excelWindow As Excel.Window)
        ContextDataString = GetSummaryFromWindow(excelWindow)
    End Sub

    Private Function GetSummaryFromApplication(ByVal excelApp As Excel._Application) As String
        Dim summary As String = Nothing
        Dim window As Excel.Window = excelApp.ActiveWindow

        If window Is Nothing Then
            WindowCaption = "(no active window)"
        Else
            summary = GetSummaryFromWindow(window)
            Marshal.ReleaseComObject(window)
        End If

        Return summary
    End Function

    Private Function GetSummaryFromWindow(ByVal excelWindow As Excel.Window) As String
        If excelWindow Is Nothing Then
            Logging.Log.Add("ContextInfo.GetSummaryFromExcelWindow", "no window provided")
            Return "(no window provided)"
        End If

        WindowType = "Excel.Window"
        Dim caption As Object = excelWindow.Caption ' sort of abnormality of the Excel object model: Window.Caption is an Object
        WindowCaption = caption.ToString()
        Dim summary As String = Nothing
        Dim sheet As Object = excelWindow.ActiveSheet

        If sheet IsNot Nothing Then

            If TypeOf sheet Is Excel.Worksheet Then
                WindowType = "Excel.Window (ActiveSheet: Worksheet)"
            End If
        End If

        Dim selection As Object = excelWindow.Selection
        Dim parent As Object = excelWindow.Parent
        Dim workbook As Excel.Workbook = TryCast(parent, Excel.Workbook)

        If workbook IsNot Nothing Then
            summary = GetSummaryFromDocument(workbook)
        End If

        summary += vbLf & GetSummaryFromSelection(selection)
        If parent IsNot Nothing Then Marshal.ReleaseComObject(parent)
        If selection IsNot Nothing Then Marshal.ReleaseComObject(selection)
        Return summary
    End Function

    Private Function GetSummaryFromSelection(ByVal excelSelection As Object) As String
        If excelSelection Is Nothing Then Return Nothing
        Dim summary As String = ""

        Try
            Dim range As Excel.Range = TryCast(excelSelection, Excel.Range)

            If range IsNot Nothing Then
                Dim rangeName As String = Nothing

                Try
                    Dim name As Excel.Name = TryCast(range.Name, Excel.Name)
                    rangeName = name.Name
                Catch
                    rangeName = "(no name)"
                End Try

                summary = "Range: Name=" & rangeName & " Address=" & range.Address & vbLf
                Return summary
            End If

            Dim shapeRange As Excel.ShapeRange = TryCast(excelSelection, Excel.ShapeRange)

            If shapeRange IsNot Nothing Then

                If shapeRange.Count = 1 Then
                    summary = "Shape: Name=" & shapeRange.Name & " Id=" + shapeRange.ID.ToString()
                    Return summary
                Else
                    summary = "ShapeRange: Count=" & shapeRange.Count
                    Return summary
                End If
            End If

            Try
                ' Does the selected object have the ShapeRange property
                shapeRange = TryCast(excelSelection.[GetType]().InvokeMember("ShapeRange", System.Reflection.BindingFlags.GetProperty, Nothing, excelSelection, Nothing), Excel.ShapeRange)
            Catch
                ' No shape selected
            End Try

            If shapeRange IsNot Nothing Then 'selected object is Rectangle, Oval, or such
                If shapeRange.Count = 1 Then
                    summary = "Shape: Name=" & shapeRange.Name & " Id=" + shapeRange.ID.ToString()
                    Return summary
                Else
                    summary = "ShapeRange: Count=" & shapeRange.Count
                    Return summary
                End If
            End If

            Dim objectTypeName As String = Microsoft.VisualBasic.Information.TypeName(excelSelection)
            summary = objectTypeName
            Dim unknownObjectName As String = Nothing

            Try
                unknownObjectName = CStr(excelSelection.[GetType]().InvokeMember("Name", System.Reflection.BindingFlags.GetProperty, Nothing, excelSelection, Nothing))
            Catch ex As Exception
                Logging.Log.Add("ContextInfo.GetSummaryFromExcelWindow. Checking an unknown object's name", ex)
                summary = objectTypeName & ": it doesn't have the Name property(?)"
            End Try

            If unknownObjectName Is Nothing Then Return summary
            Dim unknownObjectText As String = Nothing

            Try
                ' does it have text?
                ' late binding 
                unknownObjectText =
                    excelSelection.[GetType]().InvokeMember("Text", System.Reflection.BindingFlags.GetProperty, Nothing, excelSelection, Nothing).ToString()
            Catch ex As Exception
                Logging.Log.Add("ContextInfo.GetSummaryFromExcelWindow. Checking an unknown object's text", ex)
                'summary = "Unknown Object Type: it doesn't have the Text property(?)";
            End Try

            If unknownObjectText Is Nothing Then
                summary = objectTypeName & ": Name=" & unknownObjectName
            Else
                summary = objectTypeName & ": Name=" & unknownObjectName & " Text: " & unknownObjectText
                Return summary
            End If

        Catch ex As Exception
            Logging.Log.Add("ContextInfo.GetSummaryFromExcelWindow", ex)
        End Try

        Return summary
    End Function

    Private Function GetSummaryFromDocument(ByVal workbook As Excel.Workbook) As String
        Dim summary As String = "Workbook: (no workbook provided)"
        If workbook Is Nothing Then Return summary
        Return "Workbook: " & workbook.FullName
    End Function
End Class

Class ContextInfoWord
        Inherits ContextInfo

        Public Sub New(ByVal application As Word._Application)
            Dim wordApp As Word._Application = application
            ContextDataString = GetSummaryFromApplication(wordApp)
        End Sub

        Public Sub New(ByVal wordWindow As Word.Window)
            ContextDataString = GetSummaryFromWindow(wordWindow)
        End Sub

        Private Function GetSummaryFromApplication(ByVal wordApp As Word._Application) As String
            Dim windows As Word.Windows = wordApp.Windows
            Dim window As Word.Window = Nothing
            Dim summary As String = Nothing

            If windows.Count <> 0 Then
                window = wordApp.ActiveWindow
            End If

            If window IsNot Nothing Then
                summary = GetSummaryFromWindow(window)
                Marshal.ReleaseComObject(window)
            Else
                summary = "(no active window)"
            End If

            Marshal.ReleaseComObject(windows)
            Return summary
        End Function

        Private Function GetSummaryFromWindow(ByVal wordWindow As Word.Window) As String
            If wordWindow Is Nothing Then
                Logging.Log.Add("ContextInfo.GetSummaryFromWordWindow", "no window provided")
                Return "(no window provided)"
            End If

            WindowType = "Word.Window"
            WindowCaption = wordWindow.Caption
            Dim summary As String = Nothing
            Dim selection As Word.Selection = Nothing
            Dim document As Word.Document = Nothing

            Try
                document = wordWindow.Document
                summary = GetSummaryFromDocument(document)
                selection = wordWindow.Selection
                summary += vbLf & GetSummaryFromSelection(selection)
            Catch ex As Exception
                Logging.Log.Add("ContextInfo.GetSummaryFromWordWindow", ex)
            Finally
                If selection IsNot Nothing Then Marshal.ReleaseComObject(selection)
                If document IsNot Nothing Then Marshal.ReleaseComObject(document)
            End Try

            Return summary
        End Function

        Private Function GetSummaryFromSelection(ByVal selection As Word.Selection) As String
            Dim summary As String = Nothing
            Dim range As Word.Range = Nothing

            Try
                range = selection.Range

                If range.Start = range.[End] Then
                    summary = "(no text selected)"
                Else
                    summary = "Selected text=" & range.Text
                End If

            Catch ex As Exception
                Logging.Log.Add("ContextInfo.GetSummaryFromSelection(Word)", ex)
            Finally
                If range IsNot Nothing Then Marshal.ReleaseComObject(range)
            End Try

            Return summary
        End Function

        Private Function GetSummaryFromDocument(ByVal document As Word.Document) As String
            Dim summary As String = "Document: (no document provided)"
            If document Is Nothing Then Return summary
            Return "Document: " & document.FullName
        End Function
    End Class

    Class ContextInfoPowerPoint
        Inherits ContextInfo

        Public Sub New(ByVal powerPointWindow As PowerPoint.DocumentWindow)
            ContextDataString = GetSummaryFromWindow(powerPointWindow)
        End Sub

        Public Sub New(ByVal hostApp As PowerPoint._Application)
            Dim powerPointApp As PowerPoint._Application = hostApp
            ContextDataString = GetSummaryFromApplication(powerPointApp)
        End Sub

        Private Function GetSummaryFromApplication(ByVal powerPointApp As PowerPoint._Application) As String
            Dim summary As String = Nothing
            Dim window As PowerPoint.DocumentWindow = powerPointApp.ActiveWindow

            If window Is Nothing Then
                WindowCaption = "(no active window)"
            Else
                summary = GetSummaryFromWindow(window)
                Marshal.ReleaseComObject(window)
            End If

            Return summary
        End Function

        Private Function GetSummaryFromWindow(ByVal window As PowerPoint.DocumentWindow) As String
            If window Is Nothing Then
                Logging.Log.Add("ContextInfo.GetSummaryFromPowerPointWindow", "no window provided")
                Return "(no window provided)"
            End If

            WindowCaption = window.Caption
            WindowType = "PowerPoint.DocumentWindow"
            Dim selection As PowerPoint.Selection = Nothing
            Dim view As PowerPoint.View = Nothing
            Dim presentation As PowerPoint.Presentation = Nothing
            Dim summary As String = Nothing

            Try
                presentation = window.Presentation
                summary = GetSummaryFromDocument(presentation)
                selection = window.Selection
                view = window.View
                summary += vbLf & GetSummaryFromSelection(selection, view)
            Catch ex As Exception
                Logging.Log.Add("ContextInfo.GetSummaryFromPowerPointWindow. Getting selection", ex)
            Finally
                If selection IsNot Nothing Then Marshal.ReleaseComObject(selection)
                If view IsNot Nothing Then Marshal.ReleaseComObject(view)
            End Try

            Return summary
        End Function

        Private Function GetSummaryFromSelection(ByVal powerPointSelection As PowerPoint.Selection, ByVal powerPointView As PowerPoint.View) As String
            If powerPointSelection Is Nothing Then
            End If

            Dim summary As String = Nothing
            Dim slideRange As PowerPoint.SlideRange = Nothing
            Dim unknownObject As Object = Nothing

            Try
                slideRange = powerPointSelection.SlideRange

                If slideRange.Count = 1 Then
                    unknownObject = powerPointView.Slide

                    If TypeOf unknownObject Is PowerPoint.Slide Then
                        Dim slide As PowerPoint.Slide = TryCast(unknownObject, PowerPoint.Slide)
                        summary = "Current slide is " & slide.Name
                    Else
                        summary = "Can't determine the name of the current slide"
                        Dim objectTypeName As String = Microsoft.VisualBasic.Information.TypeName(unknownObject)
                        Dim message As String = String.Format("{0} - non-supported object type", objectTypeName)
                        Logging.Log.Add("PLEASE REPORT to ADX support. ContextInfo.GetSummaryFromPowerPointWindow returned this type: " & objectTypeName, "")
                    End If
                Else
                    summary = "There are " & slideRange.Count.ToString() & " slides selected"
                End If

            Catch ex As Exception
                Logging.Log.Add("ContextInfo.GetSummaryFromPowerPointWindow", ex)
            Finally
                If slideRange IsNot Nothing Then Marshal.ReleaseComObject(slideRange)
                If unknownObject IsNot Nothing Then Marshal.ReleaseComObject(unknownObject)
                If slideRange IsNot Nothing Then Marshal.ReleaseComObject(slideRange)
            End Try

            Return summary
        End Function

        Private Function GetSummaryFromDocument(ByVal presentation As PowerPoint.Presentation) As String
            Dim summary As String = "Presentation: (no presentation provided)"
            If presentation Is Nothing Then Return summary
            Return "Presentation: " & presentation.FullName
        End Function
    End Class

Class ContextInfoUnknown
    Inherits ContextInfo

    Public Sub New()
        Logging.Log.Add("ContextInfoUnknown.ctor", "Unknown or non-supported host application")
    End Sub

    Public Overrides Function ToString() As String
        Return "(unknown or non-supported host application)"
    End Function
End Class


